package tutorial;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.sanselan.ImageReadException;
import org.apache.sanselan.common.RationalNumber;
import org.apache.sanselan.formats.jpeg.JpegImageMetadata;
import org.apache.sanselan.formats.tiff.TiffField;
import org.apache.sanselan.formats.tiff.TiffImageMetadata;
import org.apache.sanselan.formats.tiff.TiffImageMetadata.GPSInfo;
import org.apache.sanselan.formats.tiff.constants.TagInfo;
import org.apache.sanselan.formats.tiff.constants.TiffConstants;

public class ExifConverter
{
public static String getAperture(JpegImageMetadata meta)
  {
   String rawAperture = extractMetadata(meta,TiffConstants.EXIF_TAG_FNUMBER);
   
   if (rawAperture == null) return null;
   
   String value;
   String[] parts = rawAperture.split("/");   
   
   try
     {
      if (parts.length > 1) value = ""+new Integer(parts[0])/new Integer(parts[1]);
      else value = parts[0];
      
      return "F/"+value;
     }
   catch (NumberFormatException e)
     {
      return rawAperture;
     }   
  }

public static String getExposureTime(JpegImageMetadata meta)
  {
   String rawExposureTime = extractMetadata(meta,TiffConstants.EXIF_TAG_EXPOSURE_TIME);

   if (rawExposureTime == null) return null;
   
   String value;
   String[] parts = rawExposureTime.split("/");   
   
   try
     {
      if (parts.length > 1) value = ""+new Integer(parts[1])/new Integer(parts[0]);
      else value = parts[0];
      
      return "1/"+value+"s";
     }
   catch (NumberFormatException e)
     {
      return rawExposureTime;
     }   
  }

public static String getFocalLength(JpegImageMetadata meta)
  {
   String rawFLength = extractMetadata(meta,TiffConstants.EXIF_TAG_FOCAL_LENGTH);   

   if (rawFLength == null) return null;
   
   String value;
   String[] parts = rawFLength.split("/");   
   
   try
     {
      if (parts.length > 1) value = ""+new Integer(parts[0])/new Integer(parts[1]);
      else value = parts[0];
      
      return value+"mm";
     }
   catch (NumberFormatException e)
     {
      return rawFLength;
     }   
  }


public static Date getCreationDate(JpegImageMetadata meta)
  {
   String rawDate = extractMetadata(meta,TiffConstants.EXIF_TAG_DATE_TIME_ORIGINAL);
   if (rawDate == null) return null;
   rawDate = rawDate.replaceAll("'","");
   
   if (rawDate == null) return new Date();
   
   try
     {
      DateFormat format = new SimpleDateFormat("yyyy:MM:dd HH:mm:ss");
      return format.parse(rawDate);      
     }
   catch (ParseException e)
     {
      return new Date();
     }   
  }

public static double getGPSLatitude(JpegImageMetadata meta)
  {
   TiffImageMetadata exif = meta.getExif();   
   if (exif == null) return 0;
         
   try
     {
      GPSInfo gpsInfo = exif.getGPS();
      
      if (gpsInfo == null) return 0;
      
      return gpsInfo.getLatitudeAsDegreesNorth();
     }
   catch (ImageReadException e)
     {
      return 0;
     }   
  }

public static double getGPSLongitude(JpegImageMetadata meta)
  {
   TiffImageMetadata exif = meta.getExif();   
   if (exif == null) return 0;
         
   try
     {
      GPSInfo gpsInfo = exif.getGPS();

      if (gpsInfo == null) return 0;
      
      return gpsInfo.getLongitudeAsDegreesEast();
     }
   catch (ImageReadException e)
     {
      return 0;
     }   
  }

protected static String extractMetadata(JpegImageMetadata metadata,TagInfo metatag)
  {
   TiffField field = metadata.findEXIFValue(metatag);
   
   if (field != null)
     {
      try
        {
         Object o = field.getValue();
         if (o instanceof RationalNumber) return ((RationalNumber)o).numerator+"/"+((RationalNumber)o).divisor;
        }
      catch (ImageReadException e)
        {
         // TODO Auto-generated catch block
         e.printStackTrace();
        }
      
      return field.getValueDescription();
     }
   
   return null;
  }
}